@@ -7,6 +7,7 @@ import hmac
 import os
 import uuid
 from datetime import datetime
+from unittest import mock
 
 import pytest
 
@@ -18,8 +19,6 @@ from zmq.eventloop.zmqstream import ZMQStream
 from jupyter_client import session as ss
 from jupyter_client import jsonutil
 
-from ipython_genutils.py3compat import string_types
-
 def _bad_packer(obj):
     raise TypeError("I don't work")
 
@@ -33,6 +32,14 @@ class SessionTestCase(BaseZMQTestCase):
         self.session = ss.Session()
 
 
+@pytest.fixture
+def no_copy_threshold():
+    """Disable zero-copy optimizations in pyzmq >= 17"""
+    with mock.patch.object(zmq, 'COPY_THRESHOLD', 1, create=True):
+        yield
+
+
+@pytest.mark.usefixtures('no_copy_threshold')
 class TestSession(SessionTestCase):
 
     def test_msg(self):
@@ -45,8 +52,8 @@ class TestSession(SessionTestCase):
         self.assertTrue(isinstance(msg['metadata'],dict))
         self.assertTrue(isinstance(msg['header'],dict))
         self.assertTrue(isinstance(msg['parent_header'],dict))
-        self.assertTrue(isinstance(msg['msg_id'], string_types))
-        self.assertTrue(isinstance(msg['msg_type'], string_types))
+        self.assertTrue(isinstance(msg['msg_id'], str))
+        self.assertTrue(isinstance(msg['msg_type'], str))
         self.assertEqual(msg['header']['msg_type'], 'execute')
         self.assertEqual(msg['msg_type'], 'execute')
 
@@ -70,7 +77,7 @@ class TestSession(SessionTestCase):
         self.assertIsInstance(self.session.auth, hmac.HMAC)
 
     def test_send(self):
-        ctx = zmq.Context.instance()
+        ctx = zmq.Context()
         A = ctx.socket(zmq.PAIR)
         B = ctx.socket(zmq.PAIR)
         A.bind("inproc://test")
@@ -123,9 +130,14 @@ class TestSession(SessionTestCase):
         self.assertEqual(new_msg['buffers'],[b'bar'])
 
         # buffers must support the buffer protocol
-        with self.assertRaises(TypeError) as cm:
+        with self.assertRaises(TypeError):
             self.session.send(A, msg, ident=b'foo', buffers=[1])
 
+        # buffers must be contiguous
+        buf = memoryview(os.urandom(16))
+        with self.assertRaises(ValueError):
+            self.session.send(A, msg, ident=b'foo', buffers=[buf[::2]])
+
         A.close()
         B.close()
         ctx.term()
@@ -135,17 +147,17 @@ class TestSession(SessionTestCase):
         s = self.session
         self.assertTrue(s.pack is ss.default_packer)
         self.assertTrue(s.unpack is ss.default_unpacker)
-        self.assertEqual(s.username, os.environ.get('USER', u'username'))
+        self.assertEqual(s.username, os.environ.get('USER', 'username'))
 
         s = ss.Session()
-        self.assertEqual(s.username, os.environ.get('USER', u'username'))
+        self.assertEqual(s.username, os.environ.get('USER', 'username'))
 
         self.assertRaises(TypeError, ss.Session, pack='hi')
         self.assertRaises(TypeError, ss.Session, unpack='hi')
         u = str(uuid.uuid4())
-        s = ss.Session(username=u'carrot', session=u)
+        s = ss.Session(username='carrot', session=u)
         self.assertEqual(s.session, u)
-        self.assertEqual(s.username, u'carrot')
+        self.assertEqual(s.username, 'carrot')
 
     def test_tracking(self):
         """test tracking messages"""
@@ -275,8 +287,8 @@ class TestSession(SessionTestCase):
         self.assertEqual(msg['parent_header'], msg2['parent_header'])
         assert isinstance(msg['content']['t'], datetime)
         assert isinstance(msg['metadata']['t'], datetime)
-        assert isinstance(msg2['content']['t'], string_types)
-        assert isinstance(msg2['metadata']['t'], string_types)
+        assert isinstance(msg2['content']['t'], str)
+        assert isinstance(msg2['metadata']['t'], str)
         self.assertEqual(msg['content'], jsonutil.extract_dates(msg2['content']))
         self.assertEqual(msg['content'], jsonutil.extract_dates(msg2['content']))
 
@@ -297,7 +309,7 @@ class TestSession(SessionTestCase):
         self._datetime_test(session)
 
     def test_send_raw(self):
-        ctx = zmq.Context.instance()
+        ctx = zmq.Context()
         A = ctx.socket(zmq.PAIR)
         B = ctx.socket(zmq.PAIR)
         A.bind("inproc://test")
@@ -320,7 +332,7 @@ class TestSession(SessionTestCase):
         A.close()
         B.close()
         ctx.term()
-    
+
     def test_clone(self):
         s = self.session
         s._add_digest('initial')
